%% DOM.m
% a script to solve the RTE in a 2D rectangular cavity using the discrete
% ordinates method
%% data initialization

%grid size
Nx = 10;
Ny = 10;

%quadrature file name
quads = 'S8.mat'; %xi, eta, w

%domain size
Lx = 1;
Ly = 1;

%grid spacing
dx = Lx/Nx;
dy = Ly/Ny;

%extinction coefficient
sigma = 3*ones(Nx,Ny);

%albedo
omega = .7;

%wall emissivity
epsw = [1 1 1 1];

%wall temperature
SB = 5.67e-8;
Tw = [2725 300 300 300];

%blackbody source

T = 300*ones(Nx,Ny);
Ib = SB/pi*T.^4;

%load the quadrature and create the intensity
load(quads); %loads xi, eta, and w (no mu since this is 2D)
N = length(xi);
I = zeros(N,Nx,Ny); %cell centered intensity
Ifx = zeros(N,Nx+1,Ny); %x-face centered intensity
Ify = zeros(N,Nx,Ny+1); %y-face centered intensity
Iiter = I; %iterate for the intensity
S = zeros(Nx,Ny); %source term

%% solution procedure

tol = 1e-6; %solution tolerance
res = 1; %initial residual
gamma = 0.5; %1 for step differencing, 0.5 for diamond differencing
residual = 1; %residual history

while (res > tol)
  
    %update the source term (in this problem, isotropic)
    S = (1-omega)*SB*sigma.*Ib;
    for i = 1:N
       S = S + omega*sigma./(4*pi)*w(i).*squeeze(Iiter(i,:,:)); 
    end
    
    %update the boundary intensities
    reflect1 = 0;
    for m = 1:N
       if xi(m) < 0
          reflect1 = reflect1 + (1-epsw(1))/pi*w(m)*abs(xi(m))*squeeze(Ifx(m,1,:)); 
       end
    end
    for m = 1:N
       if xi(m) > 0
          Ifx(m,1,:) = epsw(1)*SB*Tw(1)^4/pi + reflect1;
       end
    end
    
    reflect2 = 0;
    for m = 1:N
       if xi(m) > 0
          reflect2 = reflect2 + (1-epsw(2))/pi*w(m)*xi(m)*squeeze(Ifx(m,Nx+1,:)); 
       end
    end
    for m = 1:N
       if xi(m) < 0
          Ifx(m,Nx+1,:) = epsw(2)*SB*Tw(2)^4/pi + reflect2;
       end
    end
    
    reflect3 = 0;
    for m = 1:N
       if eta(m) < 0
          reflect3 = reflect3 + (1-epsw(3))/pi*w(m)*abs(eta(m))*squeeze(Ify(m,:,1)); 
       end
    end
    for m = 1:N
       if eta(m) > 0
          Ify(m,:,1) = epsw(3)*SB*Tw(3)^4/pi + reflect3;
       end
    end
    
    reflect4 = 0;
    for m = 1:N
       if eta(m) > 0
          reflect4 = reflect4 + (1-epsw(4))/pi*w(m)*eta(m)*squeeze(Ify(m,:,Ny+1)); 
       end
    end
    for m = 1:N
       if eta(m) < 0
          Ify(m,:,Ny+1) = epsw(4)*SB*Tw(4)^4/pi + reflect4;
       end
    end
    
    
    %sweep for the new intensity
    for m = 1:N
        %determine sweeping direction
        %if xi>0, angle points in +x, sweep from left to right
        if (xi(m) > 0)
            dindx = 1;
            startx = 1;
            endx = Nx;
        else %angle points in -x, sweep from right to left
            dindx = -1;
            startx = Nx;
            endx = 1;
        end
        if (eta(m) > 0) %angle points in +y, sweep from bottom to top
            dindy = 1;
            starty = 1;
            endy = Ny;
        else %angle points in -y, sweep from top to bottom
            dindy = -1;
            starty = Ny;
            endy = 1;
        end
        
        for j = starty:dindy:endy %from starting y point to finishing y
           
            for i = startx:dindx:endx %same for x
                
                indx = i - min(dindx,0); %index of upwind x-face intensity
                indy = j - min(dindy,0); %index of upwind y-face intensity
                I(m,i,j) = (S(i,j)*dx*dy + abs(xi(m))*dy*Ifx(m,indx,j)/gamma + ...
                    abs(eta(m))*dx*Ify(m,i,indy)/gamma)/(sigma(i,j)*dx*dy + abs(xi(m))*dy/gamma + ...
                    abs(eta(m))*dx/gamma); %integrate to compute cell-centered I
                
                %compute downwind intensities
                Ifx(m,indx+dindx,j) = (I(m,i,j)-(1-gamma)*Ifx(m,indx,j))/gamma;
                Ify(m,i,indy+dindy) = (I(m,i,j)-(1-gamma)*Ify(m,i,indy))/gamma;
             
                
            end
            
        end
        
    end
    
    %compute a residual for intensity using 2-norm
    res = sqrt(1/(Nx*Ny*N)*sum(sum(sum(((I-Iiter).^2)./I.^2))))
    residual = horzcat(residual,res);
    
    %update the iterate. I is the new guess for the intensity
    Iiter = I;
    
end


%reduce the intensity to summation over all angles
G = zeros(Nx,Ny);
for i = 1:N
   G = G+w(i)*squeeze(I(i,:,:)); 
end

figure
subplot(1,2,1)
pcolor(G')
axis xy
shading interp

% x = linspace(0,1,Nx);
% plot(x,G(:,round(Nx/2))/pi,'k','LineWidth',2);
% hold on

subplot(1,2,2)
semilogy(residual,'k','LineWidth',2)
title(strcat('spectral radius: ',num2str(residual(end)/residual(end-1))),'FontSize',20)